﻿using MoonTools.ECS;
using Nerfed.Runtime.Components;
using Nerfed.Runtime.Util;
using System.Numerics;

namespace Nerfed.Runtime.Systems
{
    public class TransformSystem : MoonTools.ECS.System
    {
        private readonly Filter rootEntitiesFilter;

        public TransformSystem(World world) : base(world)
        {
            rootEntitiesFilter = FilterBuilder.Include<LocalTransform>().Exclude<Parent>().Build();
        }

        public override void Update(TimeSpan delta)
        {
            Matrix4x4 rootMatrix = Matrix4x4.Identity;

            foreach (Entity entity in rootEntitiesFilter.Entities)
            {
                UpdateWorldTransform(entity, rootMatrix);
            }
        }

        private void UpdateWorldTransform(in Entity entity, in Matrix4x4 parentLocalToWorld)
        {
            // TODO: Only update dirty transforms.
            // Maybe store the local transform matrix.
            // If something is dirty all the children need to update their localToWorld matrix.

            LocalTransform localTransform = Get<LocalTransform>(entity);
            Matrix4x4 localToWorldMatrix = Matrix4x4.Multiply(parentLocalToWorld, localTransform.TRS());
            LocalToWorld localToWorld = new(localToWorldMatrix);
            Set(entity, localToWorld);

            Log.Info($"Entity {entity} | local position {localTransform.position} | world position {localToWorldMatrix.Translation}");

            ReverseSpanEnumerator<Entity> childEntities = World.OutRelations<ChildRelation>(entity);
            foreach (Entity childEntity in childEntities)
            {
                UpdateWorldTransform(childEntity, localToWorldMatrix);
            }
        }
    }
}